/************************************************
 * TESSY Low Level Communication Module (Slave)
 * for Cosmic Software's ZAP
 *
 * (c) 2009 Razorcat Development GmbH
 *
 * Created: 2009-03-17, bs
 ***********************************************/

#include "tslow.h"
#include "tscoml.h"


TS_SYNC_TYPE ts_sync_state;

/* IO Buffer */
uint8 ts_output_buffer[TS_BUFFER_SIZE];
uint8 ts_input_buffer[TS_BUFFER_SIZE];


static int32 InputStart = TS_BUFFER_SIZE;
static int32 OutputEnd;

// Write one character into test_out.dat
int tslows_putval(int val)
{
OUTCH:
	return(val);
}

// Read one character from test_in.dat
int tslows_getval(void)
{
	volatile int val;
INCH:
	return(val);
}

// Write error code into test_done.flg
int tslows_error(int val)
{
ERRORCH:
	return(val);
}


/*
 */
void tslows_sync(void)
{
	int counter;
	
	if (ts_sync_state == TS_SYNC_GET) {
		for (counter = 0; counter < TS_BUFFER_SIZE; counter++) {
			ts_input_buffer[counter] = (uint8)tslows_getval();
		}
	}
	else if (ts_sync_state == TS_SYNC_PUT) {
		for (counter = 0; counter < TS_BUFFER_SIZE; counter++) {
			tslows_putval(ts_output_buffer[counter]);
		}
	}
	ts_sync_state = (TS_SYNC_TYPE)TS_SYNC_NONE;
}


/* Set an Error Flag, invoke SYNC and hang up
 ************************************************************/
 
void tslow_error_abort(uint8 error_code)
{
	ts_output_buffer[0] = error_code;
	ts_sync_state       = TS_SYNC_ERROR;
	tslows_error(error_code);
	return;
}


/* Get data from master/slave communication buffer
 *
 * Input: - Pointer to memory where the data should be copied to.
 *        - Number of  bytes to transmit
 * NOTE : Memory for data must be allocated by caller.
 ******************************************************************************/

int32 tslow_get_buffer(uint8 *buffer,
							  int32 size)
{
	int32 bytesCopied;

	for (bytesCopied = 0;
		  bytesCopied < size;
		  bytesCopied++,
		  InputStart++) {

		/* Retrieve next data segment. */
		if (InputStart  == TS_BUFFER_SIZE) {
			ts_sync_state = TS_SYNC_GET;
			tslows_sync();
			InputStart    = 0;
		}
		buffer[bytesCopied] = ts_input_buffer[InputStart];
	}

	/* Retrieve next data segment. */
	if (InputStart  == TS_BUFFER_SIZE) {
		ts_sync_state = TS_SYNC_GET;
		tslows_sync();
		InputStart    = 0;
	}

	/* Return number of bytes copied. */
	return (int32)bytesCopied;	
}


/* Put data to master/slave communication buffer
 *
 * Input: - Pointer to data bytes to copy
 *        - number of bytes to transmit
 ******************************************************************************/
int8 tslow_put_buffer(uint8 *buffer,
							 int32 size)
{
	int32 bytesCopied;

	/* Check size.	*/
	if (size == 0) {
		return TS_SUCCESS;
	}

	/* Loop until all bytes are copied. */	
	for (bytesCopied = 0;
		  bytesCopied < size;
	  	  bytesCopied++,
	  	  OutputEnd++) {

		/* flush buffer if full */
		if (OutputEnd   == TS_BUFFER_SIZE) {
			ts_sync_state = TS_SYNC_PUT;
			tslows_sync();
			OutputEnd = 0;
		}

		/* Copy next byte. */
		ts_output_buffer[OutputEnd] = buffer[bytesCopied];
	}

	/* Done successfully. */
	return TS_SUCCESS;
}


/*
 */
int8 tslow_connect(void)
{
	ts_sync_state = TS_SYNC_NONE;
	InputStart    = TS_BUFFER_SIZE;
	OutputEnd     = 0;

	/* Done successfully. */
	return TS_SUCCESS;
}

int tslow_quit_zap(void)
{
	volatile int val;
END_MAIN:
	return TS_SUCCESS;
}

int8 tslow_fill_done_flag(void)
{
	volatile int val = 0;
FILLDONE:
	return TS_SUCCESS;
}

int8 tslow_close_channels(void)
{
	volatile int val;
END_COMM:
	return TS_SUCCESS;
}

int8 tslow_disconnect(void)
{
	/* 
	 * Flush output buffer.
	 */
	ts_sync_state = TS_SYNC_PUT;
	tslows_sync();

	tslow_fill_done_flag();
	tslow_close_channels();
	tslow_quit_zap();

	return TS_SUCCESS;
}
